@@ -147,7 +147,7 @@ static GstStaticPadTemplate sbc_enc_sink_factory =
 		GST_STATIC_CAPS("audio/x-raw-int, "
 				"rate = (int) { 16000, 32000, 44100, 48000 }, "
 				"channels = (int) [ 1, 2 ], "
-				"endianness = (int) LITTLE_ENDIAN, "
+				"endianness = (int) BYTE_ORDER, "
 				"signed = (boolean) true, "
 				"width = (int) 16, "
 				"depth = (int) 16"));
@@ -371,7 +371,7 @@ static GstFlowReturn sbc_enc_chain(GstPad *pad, GstBuffer *buffer)
 	gst_adapter_push(adapter, buffer);
 
 	while (gst_adapter_available(adapter) >= enc->codesize &&
-			res == GST_FLOW_OK) {
+							res == GST_FLOW_OK) {
 		GstBuffer *output;
 		GstCaps *caps;
 		const guint8 *data;
